function [res,V,PVU]  = fdimwIc(X0,param)
%MWTRAN      computes equilibria during the post-Brexit transition
%                
%            The model assumes multinationals with technology (ome) maximize
%            dividends subject to contraints on capital:
%
%               max (1-taud) sum_t p_t D_t(ome)
%
%               st  D(ome) =sum_i {(1-taup_i)[P_i(ome)(Y_i(ome)-delT K_i(ome)
%                                - X_Ii(ome)-chi_i(ome) X_M(ome)]
%                                - W_iL_i(ome)-P_i(ome)(K'_Ti(ome)-K_Ti(ome))}
%
%                   K'_Ti(ome) = (1-delT) K_Ti(ome)+X_Ti(ome)
%                   K'_Ii(ome) = (1-delI) K_Ii(ome)+X_Ii(ome)
%                   M'(ome)    = (1-delM) M(ome)   +X_M(ome) 
%                   Y_i(ome)   = A_i sig_i(ome) (N_i M(ome))^phi Z_i(ome)
%                   Z_i(ome)   = K_Ti(ome)^alpT K_Ii(ome)^alpI L_i(ome)^alpL
%
%
%            Households in country i maximixe
%
%               max sum_t beta^t (log C_it + psi log (1-L_it)) N_it
%              
%               st  sum_t p_t {(1+tauc_i) P_it [t] C_it + sum_ome V_t(ome)*
%                                     (S_it+1(ome)-S_it(ome) + B_it+1-B_it}
%                      <= sum_t p_t {(1-taul_i) W_it L_it +(1-taud) 
%                                     sum_ome S_it(ome) D_t(ome)
%                                     +rb_t B_it + Kap_it
%
%                   C_i      = CES(C_i(ome))
%                   C_i(ome) = CES(C^fdi_i(ome),C^trade_i(ome)) 
%                              if multinational with ome not from i
%    
%

%            Ellen McGrattan, 9-7-06
%            Revised, 7-30-18
%

%
% Assign fixed parameters:
%

I           = param(1);
T           = param(2);
alphI       = param(3);
alphT       = param(4);
beta        = param(5);
deltI       = param(6);
deltM       = param(7);
deltT       = param(8);
gamN        = param(9);
gamY        = param(10);
phi         = param(11);
psi         = param(12);
rho         = param(13);
varrho      = param(14);
kIdivkT     = param(15:14+I);
chiji       = zeros(I);
cjiwgt      = zeros(I);
cjifwgt     = zeros(I);
cjitwgt     = zeros(I);
chiji(:)    = param(15+I:14+I+I*I);
cjiwgt(:)   = param(15+I+I*I:14+I+2*I*I);
cjifwgt(:)  = param(15+I+2*I*I:14+I+3*I*I);
cjitwgt(:)  = param(15+I+3*I*I:14+I+4*I*I);
kT0ji       = param(15+I+4*I*I:14+I+5*I*I);
M0j         = param(15+I+5*I*I:14+2*I+5*I*I);
B0i         = param(15+2*I+5*I*I:14+3*I+5*I*I);
tauci       = param(15+3*I+5*I*I:14+4*I+5*I*I);
tauli       = param(15+4*I+5*I*I:14+5*I+5*I*I);
taudi       = param(15+5*I+5*I*I:14+6*I+5*I*I);
Trani       = param(15+6*I+5*I*I:14+7*I+5*I*I);
lnbi        = param(15+7*I+5*I*I:14+8*I+5*I*I);
xnbyi       = param(15+8*I+5*I*I:14+9*I+5*I*I);
ynbyi       = param(15+9*I+5*I*I:14+10*I+5*I*I);

%
% Assign intermediate parameters:
%

alphL       = 1-alphT-alphI;
gamy        = (1+gamY)/(1+gamN)-1;
betg        = beta/(1+gamy);
irate       = (1+gamy)/beta-1;
phi1        = 1-phi;
rho1        = 1-rho;
varrho1     = 1-varrho;
oi          = ones(I,1);
ot          = ones(T,1);
V           = zeros(T,13*I+4);

%
% Time-varying parameters:
%
i           = 14+10*I+5*I*I;
Ati         = zeros(T,I);
Nti         = zeros(T,I);
taupti      = zeros(T,I);
shrtji      = zeros(T,I*I);
sigtji      = zeros(T,I*I);
icetj2i     = zeros(T,I*I);
Ati(:)      = param(i+1:i+T*I);
Nti(:)      = param(i+1+T*I:i+2*T*I);
taupti(:)   = param(i+1+2*T*I:i+3*T*I);
i           = i+3*T*I;
shrtji(:)   = param(i+1:i+T*I*I)';
sigtji(:)   = param(i+1+T*I*I:i+2*T*I*I)';
icetj2i(:)  = param(i+1+2*T*I*I:i+3*T*I*I)';

%
% Variables to be computed in equilibrium:
%
X           = zeros(T,4*I+2*I*I-2);
X(:)        = X0;

cti         = X(:,1:I);
lti         = X(:,I+1:2*I);
Mntj        = X(:,2*I+1:3*I);
kTntji      = X(:,3*I+1:3*I+I*I);
ptji        = [ot,X(:,3*I+I*I+1:3*I+2*I*I-1)];
Bnti        = X(:,3*I+2*I*I:4*I+2*I*I-2);

%
% Current period stocks
%
kTtji       = [kT0ji';kTntji(1:T-1,:)];
Mtj         = [M0j';Mntj(1:T-1,:)];
kIdivkT     = ones(I,1)*kIdivkT';
kI0ji       = kIdivkT(:).*kT0ji;

%
% Next period variables
%
cnti        = [cti(2:T,:);cti(T,:)];
lnti        = [lti(2:T,:);lti(T,:)];
pntji       = [ptji(2:T,:);ptji(T,:)];
Anti        = [Ati(2:T,:);Ati(T,:)];
Nnti        = [Nti(2:T,:);Nti(T,:)];
taupnti     = [taupti(2:T,:);taupti(T,:)];
shrntji     = [shrtji(2:T,:);shrtji(T,:)];
signtji     = [sigtji(2:T,:);sigtji(T,:)];
icentj2i    = [icetj2i(2:T,:);icetj2i(T,:)];

%
% Bonds have to add up
%
if (I>2);
  tem       = -sum((Bnti)')';
else
  tem       = -Bnti;
end;
Bnti        = [tem,Bnti];
Bti         = [B0i';Bnti(1:T-1,:)];

%
% Intermediate variables
%

kTji        = zeros(I);
kTnji       = zeros(I);
kIji        = zeros(I);
kInji       = zeros(I);
pji         = zeros(I);
pnji        = zeros(I);
shrji       = zeros(I);
shrnji      = zeros(I);
sigji       = zeros(I);
signji      = zeros(I);
icej2i      = zeros(I);
icenj2i     = zeros(I);
rbn         = irate;
kInji(:)    = kI0ji;
PVU         = zeros(I,1);
betn        = beta*(1+gamN);

for t=1:T;

  %   * exogenous parameters
  %

  Ai        = Ati(t,:)';
  Ani       = Anti(t,:)';
  Ni        = Nti(t,:)';
  Nni       = Nnti(t,:)';
  taupi     = taupti(t,:)';
  taupni    = taupnti(t,:)';
  shrji(:)  = shrtji(t,:)';
  sigji(:)  = sigtji(t,:)';
  icej2i(:) = icetj2i(t,:)';
  shrnji(:) = shrntji(t,:)';
  signji(:) = signtji(t,:)';
  icenj2i(:)= icentj2i(t,:)';

  %   * unknowns in t,t+1
  %
  ci        = cti(t,:)';
  li        = lti(t,:)';
  Mj        = Mtj(t,:)';
  kTji(:)   = kTtji(t,:)';
  pji(:)    = ptji(t,:)';
  Bi        = Bti(t,:)';
  
  cni       = cnti(t,:)';
  lni       = lnti(t,:)';
  Mnj       = Mntj(t,:)';
  kTnji(:)  = kTntji(t,:)';
  pnji(:)   = pntji(t,:)';
  Bni       = Bnti(t,:)';

  %   * prices
  %
  pjifdi    = pji;
  pjitr     = (diag(pji)*oi').*(1+icej2i);
  pjiast    = (cjifwgt.^varrho.*pjifdi.^varrho1+cjitwgt.^varrho.*pjitr.^ ...
              varrho1).^(1/varrho1);
  tem       = cjiwgt.^rho.*pjiast.^rho1;
  tem       = tem-diag(diag(tem));
  pi        = (diag(cjiwgt).^rho.*diag(pji).^rho1 + sum(tem)').^(1/rho1);

  pnjifdi   = pnji;
  pnjitr    = (diag(pnji)*oi').*(1+icenj2i);
  pnjiast   = (cjifwgt.^varrho.*pnjifdi.^varrho1+cjitwgt.^varrho.*pnjitr.^ ...
               varrho1).^(1/varrho1);
  tem       = cjiwgt.^rho.*pnjiast.^rho1;
  tem       = tem-diag(diag(tem));
  pni       = (diag(cjiwgt).^rho.*diag(pnji).^rho1 + sum(tem)').^(1/rho1);


  %   * consumptions
  %
  cjiast    = (pjiast./(cjiwgt.*(ones(I,1)*pi'))).^(-rho).*(ones(I,1)*ci');
  cjifdi    = (pjifdi./(cjifwgt.*pjiast)).^(-varrho).*cjiast;
  cjitr     = (pjitr./(cjitwgt.*pjiast)).^(-varrho).*cjiast;
  cii       = (diag(pji)./(diag(cjiwgt).*pi)).^(-rho).*ci;

  %   * utilities
  %

  uc        = oi./ci;
  ul        = psi*oi./(oi-li-lnbi);
  ucn       = oi./cni;
  uln       = psi*oi./(oi-lni-lnbi);

  %   * interest rates
  %
  rb        = rbn;
  rbn       = pni(1)*uc(1)/(betg*pi(1)*ucn(1))-1;
  rTn       = rbn*oi./(1-taupni)+deltT;
  rIn       = (1+rbn)*(1-taupi)./(1-taupni)+deltI-1;

  %   * intangible capitals
  %
  iratio    = alphI*rTn./(alphT*rIn);
  kIji      = kInji;
  kInji     = kTnji.*(oi*iratio');

  %   * wage rates
  %
  wi        = (1+tauci).*pi.*ul./((1-tauli).*uc);
  wni       = (1+tauci).*pni.*uln./((1-tauli).*ucn);

  %   * productivities
  %
  prji      = (oi*wi')./(phi1*alphL*pji);
  prnji     = (oi*wni')./(phi1*alphL*pnji);


  %   * outputs
  %
  Aji       = sigji.*(oi*Ai');
  Anji      = signji.*(oi*Ani');
  yji       = (Aji.*kTji.^(phi1*alphT).*(kIji).^(phi1*alphI).* ...
               prji.^(-phi1*alphL).*(Mj*oi').^phi).^(1/(1-phi1*alphL));
  ynji      = (Anji.*kTnji.^(phi1*alphT).*(kInji).^(phi1*alphI).* ...
               prnji.^(-phi1*alphL).*(Mnj*oi').^phi).^(1/(1-phi1*alphL));

  %
  %   * labor inputs
  %
  lji       = yji./prji;

  %   * investments
  %
  xTji      = (1+gamY)*(oi*(Nni./Ni)').*kTnji-(1-deltT)*kTji;
  xIji      = (1+gamY)*(oi*(Nni./Ni)').*kInji-(1-deltI)*kIji;
  xMj       = (1+gamY)*Mnj-(1-deltM)*Mj;

  %   * dividends
  %
  dji       = (oi*(1-taupi')).*((oi*Ni').*(pji.*(yji-deltT*kTji- ...
              xIji)-(oi*wi').*lji)-chiji.*pji.*(xMj*oi'))- ...
              pji.*((1+gamY)*(oi*Nni').*kTnji-(oi*Ni').*kTji);
  dj        = sum(dji')';

  %   * values (assuming interiority)
  %
  vj        = (1-taudi).*(sum(((oi*Ni').*pji.*(kTnji+(oi*(1-taupi)').* ...
               kInji))')'+Mj.*sum((chiji.*pji.*(oi*(1-taupi)'))')');

  %   * transfers
  %

  gdpi      = sum(pji.*(yji-xIji-chiji.*(xMj*(oi./Ni)')))'./(1-ynbyi);
  ynbi      = ynbyi.*gdpi;
  xnbi      = xnbyi.*gdpi;
  kapi      = tauci.*pi.*ci+tauli.*wi.*li+taudi.*sum(shrji.*(dj*oi'))'./Ni+ ...
              taupi.*(sum(pji.*(yji-deltT*kTji-xIji))'-wi.*li-sum(chiji.* ...
              pji.*(xMj*oi'))'./Ni)+ynbi-xnbi-Trani./Ni;


  %   * returns to capital
  %
  rkTnji    = phi1*alphT*ynji./kTnji;
  rMnj      = sum(((oi*(1-taupni')).*pnji.*(phi*ynji.*((oi./Mnj)*Nni')+ ...
              (1-deltM)*chiji))')'./sum(((oi*(1-taupi')).*chiji.*pji)')'-1;

  %   * resource constraints
  %
  ctrade    = sum(((oi*Ni').*(1+icej2i).*cjitr./(Ni*oi'))')'- ...
              diag((1+icej2i).*cjitr);
  rcown     = diag(yji-xTji-xIji)-cii-ctrade-xMj./Ni+(ynbi-xnbi)./diag(pji);
  rc        = yji-cjifdi-xTji-xIji;
  for i=1:I;
    rc(i,i) = rcown(i);
  end;
  rc        = rc(:);

  %
  % Residual equations 
  %

  tem       = (1+rbn)*pji./pnji-1-(oi*(1-taupni)').*(rkTnji-deltT);
  res(t,:)  = [(1+rb)*Bi./Ni-(1+gamY)*Bni./Ni+(1-tauli).*wi.*li+ ...
                   (1-taudi).*sum(shrji.*(dj*oi'))'./Ni+kapi- ...
                   (1+tauci).*pi.*ci-sum((vj*oi').*(shrnji-shrji))'./Ni;
               sum(lji)'-li;
               rMnj-rbn;
               tem(:);
               rc(2:I*I);
               pni(1)*uc(1)/(pi(1)*ucn(1))- ...
                   pni(2:I).*uc(2:I)./(pi(2:I).*ucn(2:I))]';

  if t==T;
    k       = [2*I+1:3*I,3*I+1:3*I+I*I,3*I+2*I*I:4*I+2*I*I-2]'; 
    res(t,k)= [Mnj-Mj;kTnji(:)-kTji(:);Bni(2:I)-Bi(2:I)]';
  end;

  if (nargout>1);
    Nji      = oi*Ni';
    Nnji     = oi*Nni';
    GDPji    = pji.*(Nji.*(yji-xIji)-chiji.*(xMj*oi')) + diag(Ni.*ynbi);
    GDPi     = sum(GDPji)';
    Ybusji   = pji.*Nji.*yji;
    XTbusji  = pji.*Nji.*xTji;
    XIbusji  = pji.*Nji.*xIji;

    Ybusi    = sum(Ybusji)';
    XTbusi   = sum(XTbusji)';
    XIbusi   = sum(XIbusji)';
    Ytoti    = Ybusi+Ni.*ynbi;
    XTtoti   = XTbusi+Ni.*xnbi;
    Ci       = Ni.*ci;
    NXi      = GDPi-Ni.*(sum(pji.*xTji)'+pi.*ci+xnbi);
    EQPi     = sum((shrji.*(dj*oi'))')'-diag(shrji.*(dj*oi'));
    EQRi     = sum(shrji.*(dj*oi'))'-diag(shrji.*(dj*oi'));

    tem      = pji.*((1+gamY)*Nnji.*kTnji-Nji.*kTji);
    tem      = tem-diag(diag(tem));
    FDIE2U   = tem(2,3);
    FDIU2E   = tem(3,2);
    FDIrei   = sum(tem)';
    DIArei   = sum(tem')';
    DIAdivi  = sum(dji')'-diag(dji);
    FDIdivi  = sum(dji)'-diag(dji);
    DBRi     = max(rb*Bi,0);
    DBPi     = max(-rb*Bi,0);
    EXi      = sum((pjitr.*(oi*Ni').*cjitr)')'-diag(pjitr.*(oi*Ni').*cjitr);
    IMi      = sum(pjitr.*(oi*Ni').*cjitr)'-diag(pjitr.*(oi*Ni').*cjitr);
    tem      = pjitr.*(oi*Ni').*cjitr;
    TRE2U    = tem(2,3);
    TRU2E    = tem(3,2);
    NFRi     = DIAdivi+DIArei+EQRi+DBRi;
    NFPi     = FDIdivi+FDIrei+EQPi+DBPi;
    NFIci    = DIAdivi+DIArei-FDIdivi-FDIrei;
    NFIpi    = EQRi+DBRi-EQPi-DBPi;
    NXi      = EXi-IMi;
    CAi      = NXi+NFRi-NFPi-Trani;
    tem      = (vj*oi').*(shrnji-shrji);
    tem      = tem-diag(diag(tem));
    FAout    = DIArei+sum(tem)'+max((1+gamY)*Bni-Bi,0);
    FAin     = FDIrei+sum(tem')'+max(-(1+gamY)*Bni+Bi,0);
    FAi      = FAout-FAin;

    GNPi     = GDPi+NFRi-NFPi;
    V(t,:)   = [DIArei;FDIrei;EXi;IMi;Ybusi;pi.*ci;XTbusi;XIbusi;
                diag(pji).*xMj;GDPi;GNPi;wi;li;FDIE2U/GNPi(3)*100;
                FDIU2E/GNPi(2)*100;TRE2U/GNPi(3)*100;TRU2E/GNPi(2)*100];

    pus(t,:) = pji(:,1)';
    yus(t,:) = yji(:,1)';

    PVU      = PVU+betn^(t-1)*(log(ci)+psi*log(1-li-lnbi));
  end;
end;
res          = res(:);

%
% Compute chain-weighted output deflator
%
if (nargout>1);
  Def      = fisher(pus,yus,1,2);
  V        = [V,Def/Def(1)];
end;

%
% Store lifetime utility and compute the gain with a benchmark PVU0:
%  
%     gain = (exp( (1-betn)*(PVU-PVU0))-1)*100;
%
if (nargout>2);
  PVU      =  PVU+betn^T*(log(ci)+psi*log(1-li-lnbi))/(1-betn);
end;

